@@ -55,10 +55,13 @@ absl::Status StaticDataCertificateProvider::ValidateCredentials() const {
 FileWatcherCertificateProvider::FileWatcherCertificateProvider(
     const std::string& private_key_path,
     const std::string& identity_certificate_path,
-    const std::string& root_cert_path, unsigned int refresh_interval_sec) {
+    const std::string& root_cert_path,
+    const std::string& spiffe_bundle_map_path,
+    unsigned int refresh_interval_sec) {
   c_provider_ = grpc_tls_certificate_provider_file_watcher_create(
       private_key_path.c_str(), identity_certificate_path.c_str(),
-      root_cert_path.c_str(), refresh_interval_sec);
+      root_cert_path.c_str(), spiffe_bundle_map_path.c_str(),
+      refresh_interval_sec);
   CHECK_NE(c_provider_, nullptr);
 };
 
