@@ -56,8 +56,9 @@ class Test1 : public ImplementChannelFilter<Test1> {
     void OnClientToServerHalfClose() {
       history.push_back("Test1::Call::OnClientToServerHalfClose");
     }
-    void OnFinalize(const grpc_call_final_info*, Test1*) {
+    void OnFinalize(const grpc_call_final_info*, Test1* test1) {
       history.push_back("Test1::Call::OnFinalize");
+      test1->FilterMethod();
     }
     void OnServerTrailingMetadata(ServerMetadata&) {
       history.push_back("Test1::Call::OnServerTrailingMetadata");
@@ -66,6 +67,8 @@ class Test1 : public ImplementChannelFilter<Test1> {
    private:
   };
 
+  void FilterMethod() { history.push_back("Test1::FilterMethod"); }
+
   bool StartTransportOp(grpc_transport_op* op) override {
     history.push_back("Test1::StartTransportOp");
     return false;
@@ -99,14 +102,18 @@ class Test2 : public ImplementChannelFilter<Test2> {
     void OnClientToServerHalfClose() {
       history.push_back("Test2::Call::OnClientToServerHalfClose");
     }
-    void OnServerTrailingMetadata(ServerMetadata&, Test2*) {
+    void OnServerTrailingMetadata(ServerMetadata&, Test2* test2) {
       history.push_back("Test2::Call::OnServerTrailingMetadata");
+      test2->FilterMethod();
     }
+
     static inline const NoInterceptor OnFinalize;
 
    private:
   };
 
+  void FilterMethod() { history.push_back("Test2::FilterMethod"); }
+
   bool StartTransportOp(grpc_transport_op* op) override {
     history.push_back("Test2::StartTransportOp");
     return false;
@@ -133,8 +140,9 @@ class Test3 : public ImplementChannelFilter<Test3> {
     }
     static inline const NoInterceptor OnServerInitialMetadata;
     absl::StatusOr<MessageHandle> OnClientToServerMessage(MessageHandle handle,
-                                                          Test3*) {
+                                                          Test3* test3) {
       history.push_back("Test3::Call::OnClientToServerMessage");
+      test3->FilterMethod();
       return handle;
     }
     absl::StatusOr<MessageHandle> OnServerToClientMessage(MessageHandle handle,
@@ -153,6 +161,8 @@ class Test3 : public ImplementChannelFilter<Test3> {
    private:
   };
 
+  void FilterMethod() { history.push_back("Test3::FilterMethod"); }
+
   bool StartTransportOp(grpc_transport_op* op) override {
     history.push_back("Test3::StartTransportOp");
     return false;
@@ -174,11 +184,12 @@ class Test4 : public ImplementChannelFilter<Test4> {
   class Call {
    public:
     ServerMetadataHandle OnClientInitialMetadata(const ClientMetadata&,
-                                                 Test4* filter) {
+                                                 Test4* test4) {
       history.push_back("Test4::Call::OnClientInitialMetadata");
+      test4->FilterMethod();
       return nullptr;
     }
-    void OnServerInitialMetadata(ServerMetadata&, Test4* filter) {
+    void OnServerInitialMetadata(ServerMetadata&, Test4*) {
       history.push_back("Test4::Call::OnServerInitialMetadata");
     }
     static inline const NoInterceptor OnClientToServerMessage;
@@ -192,6 +203,8 @@ class Test4 : public ImplementChannelFilter<Test4> {
    private:
   };
 
+  void FilterMethod() { history.push_back("Test4::FilterMethod"); }
+
   bool StartTransportOp(grpc_transport_op* op) override {
     history.push_back("Test4::StartTransportOp");
     return true;
@@ -212,12 +225,16 @@ class Test5 : public ImplementChannelFilter<Test5> {
   }
   class Call {
    public:
-    void OnClientInitialMetadata(ClientMetadata&, Test5* filter) {
+    explicit Call(Test5* test5) { test5->CalledFromCallConstructor(); }
+    void OnClientInitialMetadata(ClientMetadata&, Test5* test5) {
       history.push_back("Test5::Call::OnClientInitialMetadata");
+      test5->FilterMethod();
     }
     void OnServerInitialMetadata(ServerMetadata&, Test5* filter) {
       history.push_back("Test5::Call::OnServerInitialMetadata");
     }
+
+    ~Call() { history.push_back("Test5::Call::CalledFromCallDestructor"); }
     static inline const NoInterceptor OnClientToServerMessage;
     static inline const NoInterceptor OnServerToClientMessage;
     static inline const NoInterceptor OnClientToServerHalfClose;
@@ -227,6 +244,12 @@ class Test5 : public ImplementChannelFilter<Test5> {
    private:
   };
 
+  void FilterMethod() { history.push_back("Test5::FilterMethod"); }
+
+  void CalledFromCallConstructor() {
+    history.push_back("Test5::CalledFromCallConstructor");
+  }
+
   bool StartTransportOp(grpc_transport_op* op) override {
     history.push_back("Test5::StartTransportOp");
     return false;
@@ -270,9 +293,9 @@ class Test6 : public ImplementChannelFilter<Test5> {
 };
 
 using TestFusedFilter =
-    FusedFilter<FilterEndpoint::kClient, Test1, Test2, Test3, Test4, Test5>;
-using TestFailedFusedFilter = FusedFilter<FilterEndpoint::kClient, Test1, Test2,
-                                          Test3, Test4, Test5, Test6>;
+    FusedFilter<FilterEndpoint::kClient, 0, Test1, Test2, Test3, Test4, Test5>;
+using TestFailedFusedFilter = FusedFilter<FilterEndpoint::kClient, 0, Test1,
+                                          Test2, Test3, Test4, Test5, Test6>;
 
 static_assert(
     !std::is_same_v<decltype(&TestFusedFilter::Call::OnClientInitialMetadata),
@@ -313,49 +336,52 @@ TEST(FusedFilterTest, ClientFilterTest) {
   absl::StatusOr<std::unique_ptr<TestFusedFilter>> filter =
       TestFusedFilter::Create(ChannelArgs(), ChannelFilter::Args());
   CHECK(filter.ok());
-  TestFusedFilter::Call call;
-  history.clear();
-  auto message = Arena::MakePooled<Message>();
-  auto server_metadata_handle = Arena::MakePooled<ServerMetadata>();
-  auto server_trailing_metadata_handle = Arena::MakePooled<ServerMetadata>();
-  auto client_metadata_handle = Arena::MakePooled<ClientMetadata>();
-  struct grpc_call_final_info info;
-  message = RunSuccessfulPromise<Message>(
-      call.OnClientToServerMessage(std::move(message), (*filter).get()));
-  RunSuccessfulPromise<Message>(
-      call.OnServerToClientMessage(std::move(message), (*filter).get()));
-  RunSuccessfulPromise<ServerMetadata>(call.OnServerInitialMetadata(
-      std::move(server_metadata_handle), (*filter).get()));
-  RunSuccessfulPromise<ClientMetadata>(call.OnClientInitialMetadata(
-      std::move(client_metadata_handle), (*filter).get()));
-  call.OnServerTrailingMetadata(*server_trailing_metadata_handle,
-                                (*filter).get());
-  call.OnClientToServerHalfClose();
-  call.OnFinalize(&info, (*filter).get());
+  {
+    TestFusedFilter::Call call((*filter).get());
+    auto message = Arena::MakePooled<Message>();
+    auto server_metadata_handle = Arena::MakePooled<ServerMetadata>();
+    auto server_trailing_metadata_handle = Arena::MakePooled<ServerMetadata>();
+    auto client_metadata_handle = Arena::MakePooled<ClientMetadata>();
+    struct grpc_call_final_info info;
+    message = RunSuccessfulPromise<Message>(
+        call.OnClientToServerMessage(std::move(message), (*filter).get()));
+    RunSuccessfulPromise<Message>(
+        call.OnServerToClientMessage(std::move(message), (*filter).get()));
+    RunSuccessfulPromise<ServerMetadata>(call.OnServerInitialMetadata(
+        std::move(server_metadata_handle), (*filter).get()));
+    RunSuccessfulPromise<ClientMetadata>(call.OnClientInitialMetadata(
+        std::move(client_metadata_handle), (*filter).get()));
+    call.OnServerTrailingMetadata(*server_trailing_metadata_handle,
+                                  (*filter).get());
+    call.OnClientToServerHalfClose();
+    call.OnFinalize(&info, (*filter).get());
+  }
   EXPECT_THAT(
       history,
-      ElementsAre("Test2::Call::OnClientToServerMessage",
-                  "Test3::Call::OnClientToServerMessage",
-                  // ServerToClientMessage execution order must be reversed.
-                  "Test3::Call::OnServerToClientMessage",
-                  "Test2::Call::OnServerToClientMessage",
-                  "Test1::Call::OnServerToClientMessage",
-                  // ServerInitialMetadata execution order must be reversed.
-                  "Test5::Call::OnServerInitialMetadata",
-                  "Test4::Call::OnServerInitialMetadata",
-                  "Test1::Call::OnClientInitialMetadata",
-                  "Test2::Call::OnClientInitialMetadata",
-                  "Test3::Call::OnClientInitialMetadata",
-                  "Test4::Call::OnClientInitialMetadata",
-                  "Test5::Call::OnClientInitialMetadata",
-                  // ServerTrailingMetadata execution order must be reversed.
-                  "Test3::Call::OnServerTrailingMetadata",
-                  "Test2::Call::OnServerTrailingMetadata",
-                  "Test1::Call::OnServerTrailingMetadata",
-                  "Test1::Call::OnClientToServerHalfClose",
-                  "Test2::Call::OnClientToServerHalfClose",
-                  "Test1::Call::OnFinalize", "Test3::Call::OnFinalize",
-                  "Test4::Call::OnFinalize"));
+      ElementsAre(
+          "Test5::CalledFromCallConstructor",
+          "Test2::Call::OnClientToServerMessage",
+          "Test3::Call::OnClientToServerMessage", "Test3::FilterMethod",
+          // ServerToClientMessage execution order must be reversed.
+          "Test3::Call::OnServerToClientMessage",
+          "Test2::Call::OnServerToClientMessage",
+          "Test1::Call::OnServerToClientMessage",
+          // ServerInitialMetadata execution order must be reversed.
+          "Test5::Call::OnServerInitialMetadata",
+          "Test4::Call::OnServerInitialMetadata",
+          "Test1::Call::OnClientInitialMetadata",
+          "Test2::Call::OnClientInitialMetadata",
+          "Test3::Call::OnClientInitialMetadata",
+          "Test4::Call::OnClientInitialMetadata", "Test4::FilterMethod",
+          "Test5::Call::OnClientInitialMetadata", "Test5::FilterMethod",
+          // ServerTrailingMetadata execution order must be reversed.
+          "Test3::Call::OnServerTrailingMetadata",
+          "Test2::Call::OnServerTrailingMetadata", "Test2::FilterMethod",
+          "Test1::Call::OnServerTrailingMetadata",
+          "Test1::Call::OnClientToServerHalfClose",
+          "Test2::Call::OnClientToServerHalfClose", "Test1::Call::OnFinalize",
+          "Test1::FilterMethod", "Test3::Call::OnFinalize",
+          "Test4::Call::OnFinalize", "Test5::Call::CalledFromCallDestructor"));
   history.clear();
   grpc_transport_op op;
   grpc_channel_info channel_info;
