@@ -18,16 +18,18 @@
 
 #include <grpc/support/alloc.h>
 #include <grpc/support/string_util.h>
-#include <gtest/gtest.h>
 
 #include <deque>
 #include <list>
 
+#include "absl/base/no_destructor.h"
 #include "absl/log/check.h"
 #include "absl/status/status.h"
 #include "gmock/gmock.h"
+#include "gtest/gtest.h"
 #include "src/core/lib/slice/slice_internal.h"
 #include "src/core/util/crash.h"
+#include "src/core/util/match.h"
 #include "src/core/util/tmpfile.h"
 #include "test/core/test_util/test_config.h"
 #include "test/core/test_util/tls_utils.h"
@@ -46,6 +48,62 @@ namespace grpc_core {
 
 namespace testing {
 
+namespace {
+const char* kGoodSpiffeBundleMapPath =
+    "test/core/credentials/transport/tls/test_data/spiffe/"
+    "client_spiffebundle.json";
+
+const char* kGoodSpiffeBundleMapPath2 =
+    "test/core/credentials/transport/tls/test_data/spiffe/"
+    "test_bundles/spiffebundle2.json";
+
+const char* kMalformedSpiffeBundleMapPath =
+    "test/core/credentials/transport/tls/test_data/spiffe/test_bundles/"
+    "spiffebundle_malformed.json";
+
+SpiffeBundleMap GetGoodSpiffeBundleMap() {
+  static const absl::NoDestructor<SpiffeBundleMap> kSpiffeBundleMap([] {
+    auto spiffe_bundle_map =
+        SpiffeBundleMap::FromFile(kGoodSpiffeBundleMapPath);
+    EXPECT_TRUE(spiffe_bundle_map.ok());
+    return *spiffe_bundle_map;
+  }());
+  return *kSpiffeBundleMap;
+}
+
+SpiffeBundleMap GetGoodSpiffeBundleMap2() {
+  static const absl::NoDestructor<SpiffeBundleMap> kSpiffeBundleMap2([] {
+    auto spiffe_bundle_map =
+        SpiffeBundleMap::FromFile(kGoodSpiffeBundleMapPath2);
+    EXPECT_TRUE(spiffe_bundle_map.ok());
+    return *spiffe_bundle_map;
+  }());
+  return *kSpiffeBundleMap2;
+}
+
+MATCHER_P2(MatchesCredentialInfo, root_matcher, identity_matcher, "") {
+  bool ok = true;
+  ok &= ::testing::ExplainMatchResult(root_matcher, arg.root_cert_info,
+                                      result_listener);
+  ok &= ::testing::ExplainMatchResult(identity_matcher, arg.key_cert_pairs,
+                                      result_listener);
+  return ok;
+}
+
+MATCHER_P(EqRootCert, cert, "") {
+  return ::testing::ExplainMatchResult(
+      ::testing::Pointee(::testing::VariantWith<std::string>(cert)), arg,
+      result_listener);
+}
+
+MATCHER_P(EqSpiffeBundleMap, map, "") {
+  return ::testing::ExplainMatchResult(
+      ::testing::Pointee(::testing::VariantWith<SpiffeBundleMap>(map)), arg,
+      result_listener);
+}
+
+}  // namespace
+
 constexpr const char* kCertName = "cert_name";
 constexpr const char* kRootError = "Unable to get latest root certificates.";
 constexpr const char* kIdentityError =
@@ -61,13 +119,14 @@ class GrpcTlsCertificateProviderTest : public ::testing::Test {
   // CredentialInfo to the cert_update_queue of state_, and check in each test
   // if the status updates are correct.
   struct CredentialInfo {
-    std::string root_certs;
     PemKeyCertPairList key_cert_pairs;
-    CredentialInfo(std::string root, PemKeyCertPairList key_cert)
-        : root_certs(std::move(root)), key_cert_pairs(std::move(key_cert)) {}
+    std::shared_ptr<RootCertInfo> root_cert_info;
+    CredentialInfo(const RootCertInfo& roots, PemKeyCertPairList key_cert)
+        : key_cert_pairs(std::move(key_cert)),
+          root_cert_info(std::make_shared<RootCertInfo>(roots)) {}
     bool operator==(const CredentialInfo& other) const {
-      return root_certs == other.root_certs &&
-             key_cert_pairs == other.key_cert_pairs;
+      return key_cert_pairs == other.key_cert_pairs &&
+             root_cert_info == other.root_cert_info;
     }
   };
 
@@ -118,18 +177,18 @@ class GrpcTlsCertificateProviderTest : public ::testing::Test {
     ~TlsCertificatesTestWatcher() override { state_->watcher = nullptr; }
 
     void OnCertificatesChanged(
-        std::optional<absl::string_view> root_certs,
+        std::shared_ptr<RootCertInfo> roots,
         std::optional<PemKeyCertPairList> key_cert_pairs) override {
       MutexLock lock(&state_->mu);
-      std::string updated_root;
-      if (root_certs.has_value()) {
-        updated_root = std::string(*root_certs);
+      RootCertInfo updated_root;
+      if (roots != nullptr) {
+        updated_root = *roots;
       }
       PemKeyCertPairList updated_identity;
       if (key_cert_pairs.has_value()) {
         updated_identity = std::move(*key_cert_pairs);
       }
-      state_->cert_update_queue.emplace_back(std::move(updated_root),
+      state_->cert_update_queue.emplace_back(updated_root,
                                              std::move(updated_identity));
     }
 
@@ -162,6 +221,10 @@ class GrpcTlsCertificateProviderTest : public ::testing::Test {
     private_key_2_ = GetFileContents(SERVER_KEY_PATH_2);
     malformed_cert_ = GetFileContents(MALFORMED_CERT_PATH);
     malformed_key_ = GetFileContents(MALFORMED_KEY_PATH);
+    spiffe_bundle_contents_ = GetFileContents(kGoodSpiffeBundleMapPath);
+    spiffe_bundle_contents_2_ = GetFileContents(kGoodSpiffeBundleMapPath2);
+    malformed_spiffe_bundle_contents_ =
+        GetFileContents(kMalformedSpiffeBundleMapPath);
   }
 
   WatcherState* MakeWatcher(
@@ -197,6 +260,9 @@ class GrpcTlsCertificateProviderTest : public ::testing::Test {
   std::string cert_chain_2_;
   std::string malformed_cert_;
   std::string malformed_key_;
+  std::string spiffe_bundle_contents_;
+  std::string spiffe_bundle_contents_2_;
+  std::string malformed_spiffe_bundle_contents_;
   RefCountedPtr<grpc_tls_certificate_distributor> distributor_;
   // Use a std::list<> here to avoid the address invalidation caused by internal
   // reallocation of std::vector<>.
@@ -211,24 +277,26 @@ TEST_F(GrpcTlsCertificateProviderTest, StaticDataCertificateProviderCreation) {
   // Watcher watching both root and identity certs.
   WatcherState* watcher_state_1 =
       MakeWatcher(provider.distributor(), kCertName, kCertName);
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_, MakeCertKeyPairs(private_key_.c_str(),
-                                               cert_chain_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
   CancelWatch(watcher_state_1);
   // Watcher watching only root certs.
   WatcherState* watcher_state_2 =
       MakeWatcher(provider.distributor(), kCertName, std::nullopt);
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(root_cert_, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(root_cert_), PemKeyCertPairList())));
   CancelWatch(watcher_state_2);
   // Watcher watching only identity certs.
   WatcherState* watcher_state_3 =
       MakeWatcher(provider.distributor(), std::nullopt, kCertName);
-  EXPECT_THAT(
-      watcher_state_3->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          "", MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
+  EXPECT_THAT(watcher_state_3->GetCredentialQueue(),
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(""), MakeCertKeyPairs(private_key_.c_str(),
+                                                   cert_chain_.c_str()))));
   CancelWatch(watcher_state_3);
 }
 
@@ -272,42 +340,47 @@ TEST_F(GrpcTlsCertificateProviderTest,
 TEST_F(GrpcTlsCertificateProviderTest,
        FileWatcherCertificateProviderWithGoodPaths) {
   FileWatcherCertificateProvider provider(SERVER_KEY_PATH, SERVER_CERT_PATH,
-                                          CA_CERT_PATH, 1);
+                                          CA_CERT_PATH,
+                                          /*spiffe_bundle_map_path=*/"", 1);
   // Watcher watching both root and identity certs.
   WatcherState* watcher_state_1 =
       MakeWatcher(provider.distributor(), kCertName, kCertName);
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_, MakeCertKeyPairs(private_key_.c_str(),
-                                               cert_chain_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
   CancelWatch(watcher_state_1);
   // Watcher watching only root certs.
   WatcherState* watcher_state_2 =
       MakeWatcher(provider.distributor(), kCertName, std::nullopt);
   EXPECT_THAT(watcher_state_2->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(root_cert_, {})));
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(root_cert_), PemKeyCertPairList())));
   CancelWatch(watcher_state_2);
   // Watcher watching only identity certs.
   WatcherState* watcher_state_3 =
       MakeWatcher(provider.distributor(), std::nullopt, kCertName);
-  EXPECT_THAT(
-      watcher_state_3->GetCredentialQueue(),
-      ::testing::ElementsAre(CredentialInfo(
-          "", MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
+  EXPECT_THAT(watcher_state_3->GetCredentialQueue(),
+              ::testing::ElementsAre(MatchesCredentialInfo(
+                  EqRootCert(""), MakeCertKeyPairs(private_key_.c_str(),
+                                                   cert_chain_.c_str()))));
   CancelWatch(watcher_state_3);
 }
 
 TEST_F(GrpcTlsCertificateProviderTest,
        FileWatcherCertificateProviderWithGoodPathsAndCredentialValidation) {
   FileWatcherCertificateProvider provider(SERVER_KEY_PATH, SERVER_CERT_PATH,
-                                          CA_CERT_PATH, 1);
+                                          CA_CERT_PATH,
+                                          /*spiffe_bundle_map_path=*/"", 1);
   EXPECT_EQ(provider.ValidateCredentials(), absl::OkStatus());
 }
 
 TEST_F(GrpcTlsCertificateProviderTest,
        FileWatcherCertificateProviderWithMalformedRootCertificate) {
   FileWatcherCertificateProvider provider(SERVER_KEY_PATH_2, SERVER_CERT_PATH_2,
-                                          MALFORMED_CERT_PATH, 1);
+                                          MALFORMED_CERT_PATH,
+                                          /*spiffe_bundle_map_path=*/"", 1);
   EXPECT_EQ(provider.ValidateCredentials(),
             absl::FailedPreconditionError(
                 "Failed to parse root certificates as PEM: Invalid PEM."));
@@ -315,8 +388,9 @@ TEST_F(GrpcTlsCertificateProviderTest,
 
 TEST_F(GrpcTlsCertificateProviderTest,
        FileWatcherCertificateProviderWithMalformedIdentityCertificate) {
-  FileWatcherCertificateProvider provider(
-      SERVER_KEY_PATH_2, MALFORMED_CERT_PATH, CA_CERT_PATH_2, 1);
+  FileWatcherCertificateProvider provider(SERVER_KEY_PATH_2,
+                                          MALFORMED_CERT_PATH, CA_CERT_PATH_2,
+                                          /*spiffe_bundle_map_path=*/"", 1);
   EXPECT_EQ(provider.ValidateCredentials(),
             absl::FailedPreconditionError(
                 "Failed to parse certificate chain as PEM: Invalid PEM."));
@@ -324,8 +398,9 @@ TEST_F(GrpcTlsCertificateProviderTest,
 
 TEST_F(GrpcTlsCertificateProviderTest,
        FileWatcherCertificateProviderWithMalformedIdentityKey) {
-  FileWatcherCertificateProvider provider(
-      MALFORMED_KEY_PATH, SERVER_CERT_PATH_2, CA_CERT_PATH_2, 1);
+  FileWatcherCertificateProvider provider(MALFORMED_KEY_PATH,
+                                          SERVER_CERT_PATH_2, CA_CERT_PATH_2,
+                                          /*spiffe_bundle_map_path=*/"", 1);
   EXPECT_EQ(provider.ValidateCredentials(),
             absl::NotFoundError(
                 "Failed to parse private key as PEM: No private key found."));
@@ -334,7 +409,8 @@ TEST_F(GrpcTlsCertificateProviderTest,
 TEST_F(GrpcTlsCertificateProviderTest,
        FileWatcherCertificateProviderWithBadPaths) {
   FileWatcherCertificateProvider provider(INVALID_PATH, INVALID_PATH,
-                                          INVALID_PATH, 1);
+                                          INVALID_PATH,
+                                          /*spiffe_bundle_map_path=*/"", 1);
   // Watcher watching both root and identity certs.
   WatcherState* watcher_state_1 =
       MakeWatcher(provider.distributor(), kCertName, kCertName);
@@ -367,16 +443,17 @@ TEST_F(GrpcTlsCertificateProviderTest,
   TmpFile tmp_identity_key(private_key_);
   TmpFile tmp_identity_cert(cert_chain_);
   // Create FileWatcherCertificateProvider.
-  FileWatcherCertificateProvider provider(tmp_identity_key.name(),
-                                          tmp_identity_cert.name(),
-                                          tmp_root_cert.name(), 1);
+  FileWatcherCertificateProvider provider(
+      tmp_identity_key.name(), tmp_identity_cert.name(), tmp_root_cert.name(),
+      /*spiffe_bundle_map_path=*/"", 1);
   WatcherState* watcher_state_1 =
       MakeWatcher(provider.distributor(), kCertName, kCertName);
   // Expect to see the credential data.
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_, MakeCertKeyPairs(private_key_.c_str(),
-                                               cert_chain_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
   // Copy new data to files.
   // TODO(ZhenLian): right now it is not completely atomic. Use the real atomic
   // update when the directory renaming is added in gpr.
@@ -387,10 +464,11 @@ TEST_F(GrpcTlsCertificateProviderTest,
   gpr_sleep_until(gpr_time_add(gpr_now(GPR_CLOCK_MONOTONIC),
                                gpr_time_from_seconds(2, GPR_TIMESPAN)));
   // Expect to see the new credential data.
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_2_, MakeCertKeyPairs(private_key_2_.c_str(),
-                                                 cert_chain_2_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_2_),
+          MakeCertKeyPairs(private_key_2_.c_str(), cert_chain_2_.c_str()))));
   // Clean up.
   CancelWatch(watcher_state_1);
 }
@@ -402,16 +480,17 @@ TEST_F(GrpcTlsCertificateProviderTest,
   TmpFile tmp_identity_key(private_key_);
   TmpFile tmp_identity_cert(cert_chain_);
   // Create FileWatcherCertificateProvider.
-  FileWatcherCertificateProvider provider(tmp_identity_key.name(),
-                                          tmp_identity_cert.name(),
-                                          tmp_root_cert.name(), 1);
+  FileWatcherCertificateProvider provider(
+      tmp_identity_key.name(), tmp_identity_cert.name(), tmp_root_cert.name(),
+      /*spiffe_bundle_map_path=*/"", 1);
   WatcherState* watcher_state_1 =
       MakeWatcher(provider.distributor(), kCertName, kCertName);
   // Expect to see the credential data.
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_, MakeCertKeyPairs(private_key_.c_str(),
-                                               cert_chain_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
   // Copy new data to files.
   // TODO(ZhenLian): right now it is not completely atomic. Use the real atomic
   // update when the directory renaming is added in gpr.
@@ -420,10 +499,11 @@ TEST_F(GrpcTlsCertificateProviderTest,
   gpr_sleep_until(gpr_time_add(gpr_now(GPR_CLOCK_MONOTONIC),
                                gpr_time_from_seconds(2, GPR_TIMESPAN)));
   // Expect to see the new credential data.
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_2_, MakeCertKeyPairs(private_key_.c_str(),
-                                                 cert_chain_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_2_),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
   // Clean up.
   CancelWatch(watcher_state_1);
 }
@@ -435,16 +515,17 @@ TEST_F(GrpcTlsCertificateProviderTest,
   TmpFile tmp_identity_key(private_key_);
   TmpFile tmp_identity_cert(cert_chain_);
   // Create FileWatcherCertificateProvider.
-  FileWatcherCertificateProvider provider(tmp_identity_key.name(),
-                                          tmp_identity_cert.name(),
-                                          tmp_root_cert.name(), 1);
+  FileWatcherCertificateProvider provider(
+      tmp_identity_key.name(), tmp_identity_cert.name(), tmp_root_cert.name(),
+      /*spiffe_bundle_map_path=*/"", 1);
   WatcherState* watcher_state_1 =
       MakeWatcher(provider.distributor(), kCertName, kCertName);
   // Expect to see the credential data.
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_, MakeCertKeyPairs(private_key_.c_str(),
-                                               cert_chain_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
   // Copy new data to files.
   // TODO(ZhenLian): right now it is not completely atomic. Use the real atomic
   // update when the directory renaming is added in gpr.
@@ -454,10 +535,11 @@ TEST_F(GrpcTlsCertificateProviderTest,
   gpr_sleep_until(gpr_time_add(gpr_now(GPR_CLOCK_MONOTONIC),
                                gpr_time_from_seconds(2, GPR_TIMESPAN)));
   // Expect to see the new credential data.
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_, MakeCertKeyPairs(private_key_2_.c_str(),
-                                               cert_chain_2_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_),
+          MakeCertKeyPairs(private_key_2_.c_str(), cert_chain_2_.c_str()))));
   // Clean up.
   CancelWatch(watcher_state_1);
 }
@@ -469,17 +551,18 @@ TEST_F(GrpcTlsCertificateProviderTest,
   auto tmp_identity_key = std::make_unique<TmpFile>(private_key_);
   auto tmp_identity_cert = std::make_unique<TmpFile>(cert_chain_);
   // Create FileWatcherCertificateProvider.
-  FileWatcherCertificateProvider provider(tmp_identity_key->name(),
-                                          tmp_identity_cert->name(),
-                                          tmp_root_cert->name(), 1);
+  FileWatcherCertificateProvider provider(
+      tmp_identity_key->name(), tmp_identity_cert->name(),
+      tmp_root_cert->name(), /*spiffe_bundle_map_path=*/"", 1);
   WatcherState* watcher_state_1 =
       MakeWatcher(provider.distributor(), kCertName, kCertName);
   // The initial data is all good, so we expect to have successful credential
   // updates.
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_, MakeCertKeyPairs(private_key_.c_str(),
-                                               cert_chain_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
   // Delete TmpFile objects, which will remove the corresponding files.
   tmp_root_cert.reset();
   tmp_identity_key.reset();
@@ -503,17 +586,18 @@ TEST_F(GrpcTlsCertificateProviderTest,
   TmpFile tmp_identity_key(private_key_);
   TmpFile tmp_identity_cert(cert_chain_);
   // Create FileWatcherCertificateProvider.
-  FileWatcherCertificateProvider provider(tmp_identity_key.name(),
-                                          tmp_identity_cert.name(),
-                                          tmp_root_cert->name(), 1);
+  FileWatcherCertificateProvider provider(
+      tmp_identity_key.name(), tmp_identity_cert.name(), tmp_root_cert->name(),
+      /*spiffe_bundle_map_path=*/"", 1);
   WatcherState* watcher_state_1 =
       MakeWatcher(provider.distributor(), kCertName, kCertName);
   // The initial data is all good, so we expect to have successful credential
   // updates.
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_, MakeCertKeyPairs(private_key_.c_str(),
-                                               cert_chain_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
   // Delete root TmpFile object, which will remove the corresponding file.
   tmp_root_cert.reset();
   // Wait 2 seconds for the provider's refresh thread to read the deleted files.
@@ -535,17 +619,18 @@ TEST_F(GrpcTlsCertificateProviderTest,
   auto tmp_identity_key = std::make_unique<TmpFile>(private_key_);
   auto tmp_identity_cert = std::make_unique<TmpFile>(cert_chain_);
   // Create FileWatcherCertificateProvider.
-  FileWatcherCertificateProvider provider(tmp_identity_key->name(),
-                                          tmp_identity_cert->name(),
-                                          tmp_root_cert.name(), 1);
+  FileWatcherCertificateProvider provider(
+      tmp_identity_key->name(), tmp_identity_cert->name(), tmp_root_cert.name(),
+      /*spiffe_bundle_map_path=*/"", 1);
   WatcherState* watcher_state_1 =
       MakeWatcher(provider.distributor(), kCertName, kCertName);
   // The initial data is all good, so we expect to have successful credential
   // updates.
-  EXPECT_THAT(watcher_state_1->GetCredentialQueue(),
-              ::testing::ElementsAre(CredentialInfo(
-                  root_cert_, MakeCertKeyPairs(private_key_.c_str(),
-                                               cert_chain_.c_str()))));
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqRootCert(root_cert_),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
   // Delete identity TmpFile objects, which will remove the corresponding files.
   tmp_identity_key.reset();
   tmp_identity_cert.reset();
@@ -564,7 +649,8 @@ TEST_F(GrpcTlsCertificateProviderTest,
 TEST_F(GrpcTlsCertificateProviderTest,
        FileWatcherCertificateProviderTooShortRefreshIntervalIsOverwritten) {
   FileWatcherCertificateProvider provider(SERVER_KEY_PATH, SERVER_CERT_PATH,
-                                          CA_CERT_PATH, 0);
+                                          CA_CERT_PATH,
+                                          /*spiffe_bundle_map_path=*/"", 0);
   ASSERT_THAT(provider.TestOnlyGetRefreshIntervalSecond(), 1);
 }
 
@@ -616,6 +702,214 @@ TEST_F(GrpcTlsCertificateProviderTest, FailedKeyCertMatchOnInvalidPair) {
   EXPECT_FALSE(*status);
 }
 
+TEST_F(GrpcTlsCertificateProviderTest,
+       SpiffeFileWatcherCertificateProviderWithGoodPaths) {
+  FileWatcherCertificateProvider provider(SERVER_KEY_PATH, SERVER_CERT_PATH,
+                                          CA_CERT_PATH,
+                                          kGoodSpiffeBundleMapPath, 1);
+  // Watcher watching both root and identity certs.
+  WatcherState* watcher_state_1 =
+      MakeWatcher(provider.distributor(), kCertName, kCertName);
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetGoodSpiffeBundleMap()),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
+  CancelWatch(watcher_state_1);
+  // Watcher watching only root certs.
+  WatcherState* watcher_state_2 =
+      MakeWatcher(provider.distributor(), kCertName, std::nullopt);
+  EXPECT_THAT(
+      watcher_state_2->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetGoodSpiffeBundleMap()), PemKeyCertPairList())));
+  CancelWatch(watcher_state_2);
+}
+
+TEST_F(
+    GrpcTlsCertificateProviderTest,
+    SpiffeFileWatcherCertificateProviderWithGoodPathsAndCredentialValidation) {
+  FileWatcherCertificateProvider provider(SERVER_KEY_PATH, SERVER_CERT_PATH,
+                                          CA_CERT_PATH,
+                                          kGoodSpiffeBundleMapPath, 1);
+  EXPECT_EQ(provider.ValidateCredentials(), absl::OkStatus());
+}
+
+TEST_F(GrpcTlsCertificateProviderTest,
+       SpiffeFileWatcherCertificateProviderWithMissingSpiffeBundlePath) {
+  FileWatcherCertificateProvider provider(SERVER_KEY_PATH_2, SERVER_CERT_PATH_2,
+                                          CA_CERT_PATH, INVALID_PATH, 1);
+  EXPECT_EQ(provider.ValidateCredentials(),
+            absl::InvalidArgumentError(
+                "spiffe bundle map file invalid/path failed to load: INTERNAL: "
+                "Failed to load file: invalid/path due to error(fdopen): No "
+                "such file or directory"));
+}
+
+TEST_F(GrpcTlsCertificateProviderTest,
+       SpiffeFileWatcherCertificateProviderWithMalformedSpiffeBundlePath) {
+  FileWatcherCertificateProvider provider(SERVER_KEY_PATH_2, SERVER_CERT_PATH_2,
+                                          CA_CERT_PATH,
+                                          kMalformedSpiffeBundleMapPath, 1);
+  EXPECT_EQ(
+      provider.ValidateCredentials(),
+      absl::InvalidArgumentError(
+          "spiffe bundle map file "
+          "test/core/credentials/transport/tls/test_data/spiffe/test_bundles/"
+          "spiffebundle_malformed.json failed to load: INVALID_ARGUMENT: "
+          "errors validating JSON: [field: error:is not an object]"));
+}
+
+// The following tests write credential data to temporary files to test the
+// transition behavior of the provider.
+TEST_F(GrpcTlsCertificateProviderTest,
+       SpiffeFileWatcherCertificateProviderOnBothRefreshed) {
+  // Create temporary files and copy cert data into them.
+  TmpFile tmp_identity_key(private_key_);
+  TmpFile tmp_identity_cert(cert_chain_);
+  TmpFile tmp_spiffe_bundle_map(spiffe_bundle_contents_);
+  // Create FileWatcherCertificateProvider.
+  FileWatcherCertificateProvider provider(
+      tmp_identity_key.name(), tmp_identity_cert.name(),
+      /*root_cert_path=*/"", tmp_spiffe_bundle_map.name(),
+      /*refresh_interval_sec=*/1);
+  WatcherState* watcher_state_1 =
+      MakeWatcher(provider.distributor(), kCertName, kCertName);
+  // Expect to see the credential data.
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetGoodSpiffeBundleMap()),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
+  // Copy new data to files.
+  // TODO(ZhenLian): right now it is not completely atomic. Use the real atomic
+  // update when the directory renaming is added in gpr.
+  tmp_identity_key.RewriteFile(private_key_2_);
+  tmp_identity_cert.RewriteFile(cert_chain_2_);
+  tmp_spiffe_bundle_map.RewriteFile(spiffe_bundle_contents_2_);
+  // Wait 2 seconds for the provider's refresh thread to read the updated files.
+  gpr_sleep_until(gpr_time_add(gpr_now(GPR_CLOCK_MONOTONIC),
+                               gpr_time_from_seconds(2, GPR_TIMESPAN)));
+  // Expect to see the new credential data.
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetGoodSpiffeBundleMap2()),
+          MakeCertKeyPairs(private_key_2_.c_str(), cert_chain_2_.c_str()))));
+  // Clean up.
+  CancelWatch(watcher_state_1);
+}
+
+TEST_F(GrpcTlsCertificateProviderTest,
+       SpiffeFileWatcherCertificateProviderOnSpiffeBundleMapRefreshed) {
+  // Create temporary files and copy cert data into them.
+  TmpFile tmp_identity_key(private_key_);
+  TmpFile tmp_identity_cert(cert_chain_);
+  TmpFile tmp_spiffe_bundle_map(spiffe_bundle_contents_);
+  // Create FileWatcherCertificateProvider.
+  FileWatcherCertificateProvider provider(
+      tmp_identity_key.name(), tmp_identity_cert.name(),
+      /*root_cert_path=*/"", tmp_spiffe_bundle_map.name(),
+      /*refresh_interval_sec=*/1);
+  WatcherState* watcher_state_1 =
+      MakeWatcher(provider.distributor(), kCertName, kCertName);
+  // Expect to see the credential data.
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetGoodSpiffeBundleMap()),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
+  // Copy new data to files.
+  // TODO(ZhenLian): right now it is not completely atomic. Use the real
+  // atomic update when the directory renaming is added in gpr.
+  tmp_spiffe_bundle_map.RewriteFile(spiffe_bundle_contents_2_);
+  // Wait 2 seconds for the provider's refresh thread to read the updated files.
+  gpr_sleep_until(gpr_time_add(gpr_now(GPR_CLOCK_MONOTONIC),
+                               gpr_time_from_seconds(2, GPR_TIMESPAN)));
+  // Expect to see the new credential data.
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetGoodSpiffeBundleMap2()),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
+  // Clean up.
+  CancelWatch(watcher_state_1);
+}
+
+TEST_F(
+    GrpcTlsCertificateProviderTest,
+    SpiffeFileWatcherCertificateProviderWithGoodAtFirstThenDeletedSpiffeBundleMap) {
+  // Create temporary files and copy cert data into it.
+  auto tmp_spiffe_bundle_map =
+      std::make_unique<TmpFile>(spiffe_bundle_contents_);
+  TmpFile tmp_identity_key(private_key_);
+  TmpFile tmp_identity_cert(cert_chain_);
+  // Create FileWatcherCertificateProvider.
+  FileWatcherCertificateProvider provider(
+      tmp_identity_key.name(), tmp_identity_cert.name(),
+      /*root_cert_path=*/"", tmp_spiffe_bundle_map->name(),
+      /*refresh_interval_sec=*/1);
+  WatcherState* watcher_state_1 =
+      MakeWatcher(provider.distributor(), kCertName, kCertName);
+  // The initial data is all good, so we expect to have successful credential
+  // updates.
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetGoodSpiffeBundleMap()),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
+  // Delete root TmpFile object, which will remove the corresponding file.
+  tmp_spiffe_bundle_map.reset();
+  // Wait 2 seconds for the provider's refresh thread to read the deleted files.
+  gpr_sleep_until(gpr_time_add(gpr_now(GPR_CLOCK_MONOTONIC),
+                               gpr_time_from_seconds(2, GPR_TIMESPAN)));
+  // Expect to see errors sent to watchers, and no credential updates.
+  // We have no ideas on how many errors we will receive, so we only check once.
+  EXPECT_THAT(watcher_state_1->GetErrorQueue(),
+              ::testing::Contains(ErrorInfo(kRootError, "")));
+  EXPECT_THAT(watcher_state_1->GetCredentialQueue(), ::testing::ElementsAre());
+  // Clean up.
+  CancelWatch(watcher_state_1);
+}
+
+TEST_F(
+    GrpcTlsCertificateProviderTest,
+    SpiffeFileWatcherCertificateProviderWithGoodAtFirstThenDeletedBothCertsAndSpiffe) {
+  // Create temporary files and copy cert data into it.
+  auto tmp_spiffe_bundle_map =
+      std::make_unique<TmpFile>(spiffe_bundle_contents_);
+  auto tmp_identity_key = std::make_unique<TmpFile>(private_key_);
+  auto tmp_identity_cert = std::make_unique<TmpFile>(cert_chain_);
+  // Create FileWatcherCertificateProvider.
+  FileWatcherCertificateProvider provider(
+      tmp_identity_key->name(), tmp_identity_cert->name(),
+      /*root_cert_path=*/"", tmp_spiffe_bundle_map->name(),
+      /*refresh_interval_sec=*/1);
+  WatcherState* watcher_state_1 =
+      MakeWatcher(provider.distributor(), kCertName, kCertName);
+  // The initial data is all good, so we expect to have successful credential
+  // updates.
+  EXPECT_THAT(
+      watcher_state_1->GetCredentialQueue(),
+      ::testing::ElementsAre(MatchesCredentialInfo(
+          EqSpiffeBundleMap(GetGoodSpiffeBundleMap()),
+          MakeCertKeyPairs(private_key_.c_str(), cert_chain_.c_str()))));
+  // Delete TmpFile objects, which will remove the corresponding files.
+  tmp_spiffe_bundle_map.reset();
+  tmp_identity_key.reset();
+  tmp_identity_cert.reset();
+  // Wait 2 seconds for the provider's refresh thread to read the deleted files.
+  gpr_sleep_until(gpr_time_add(gpr_now(GPR_CLOCK_MONOTONIC),
+                               gpr_time_from_seconds(2, GPR_TIMESPAN)));
+  // Expect to see errors sent to watchers, and no credential updates.
+  // We have no ideas on how many errors we will receive, so we only check once.
+  EXPECT_THAT(watcher_state_1->GetErrorQueue(),
+              ::testing::Contains(ErrorInfo(kRootError, kIdentityError)));
+  EXPECT_THAT(watcher_state_1->GetCredentialQueue(), ::testing::ElementsAre());
+  // Clean up.
+  CancelWatch(watcher_state_1);
+}
+
 }  // namespace testing
 }  // namespace grpc_core
 
