@@ -442,17 +442,6 @@ std::vector<CoreTestConfiguration> End2endTestConfigs() {
           [](const ChannelArgs&, const ChannelArgs&) {
             return std::make_unique<SockpairWithMinstackFixture>(ChannelArgs());
           }},
-      CoreTestConfiguration{
-          /*name=*/"Ph2Client",
-          /*feature_mask=*/FEATURE_MASK_SUPPORTS_CLIENT_CHANNEL |
-              FEATURE_MASK_IS_HTTP2 | FEATURE_MASK_IS_CALL_V3 |
-              FEATURE_MASK_IS_PH2_CLIENT,
-          /*overridden_call_host=*/nullptr,
-          /*create_fixture=*/
-          [](const ChannelArgs& /*client_args*/,
-             const ChannelArgs& /*server_args*/) {
-            return std::make_unique<InsecureFixture>();
-          }},
   };
 }
 
