@@ -15,6 +15,9 @@
 #ifndef GRPC_TEST_CORE_TEST_UTIL_POSTMORTEM_H
 #define GRPC_TEST_CORE_TEST_UTIL_POSTMORTEM_H
 
+#include "gtest/gtest.h"
+#include "test/core/test_util/postmortem_emit.h"
+
 namespace grpc_core {
 
 // Helper class to dump useful post-mortem analysis in the event of a test
@@ -23,11 +26,10 @@ class PostMortem {
  public:
   // Used as a scoped object PostMortem will check gtest failure and emit
   // state if it sees a test failure on destruction.
-  ~PostMortem();
-
-  // Additionally, the library can be used to directly emit state in the event
-  // of a crash, for instance.
-  static void Emit();
+  ~PostMortem() {
+    if (!::testing::Test::HasFailure()) return;
+    PostMortemEmit();
+  }
 };
 
 }  // namespace grpc_core
