@@ -54,7 +54,7 @@
 #include "src/core/lib/iomgr/ev_posix.h"
 #endif  // GRPC_POSIX_SOCKET_EV
 
-#include <gtest/gtest.h>
+#include "gtest/gtest.h"
 
 using grpc::testing::EchoRequest;
 using grpc::testing::EchoResponse;
