@@ -28,7 +28,6 @@
 #include <grpcpp/health_check_service_interface.h>
 #include <grpcpp/server.h>
 #include <grpcpp/server_builder.h>
-#include <gtest/gtest.h>
 
 #include <algorithm>
 #include <condition_variable>
@@ -39,6 +38,7 @@
 
 #include "absl/log/log.h"
 #include "absl/memory/memory.h"
+#include "gtest/gtest.h"
 #include "src/core/util/backoff.h"
 #include "src/core/util/crash.h"
 #include "src/core/util/env.h"
