@@ -35,19 +35,12 @@
 
 #include "absl/log/check.h"
 #include "absl/log/log.h"
-#include "absl/memory/memory.h"
-#include "src/core/util/crash.h"
 #include "src/core/util/host_port.h"
-#include "src/core/util/latent_see.h"
-#include "src/core/util/notification.h"
-#include "src/core/util/thd.h"
 #include "src/proto/grpc/testing/worker_service.grpc.pb.h"
 #include "test/core/test_util/grpc_profiler.h"
-#include "test/core/test_util/histogram.h"
 #include "test/cpp/qps/client.h"
 #include "test/cpp/qps/qps_server_builder.h"
 #include "test/cpp/qps/server.h"
-#include "test/cpp/util/create_test_channel.h"
 #include "test/cpp/util/test_credentials_provider.h"
 
 namespace grpc {
@@ -104,94 +97,6 @@ class ScopedProfile final {
   const bool enable_;
 };
 
-#ifdef GRPC_ENABLE_LATENT_SEE
-class ScopedLatentSee final {
- public:
-  explicit ScopedLatentSee(std::string role) : role_(role) {}
-
-  ~ScopedLatentSee() {
-    CollectManager();
-    for (auto& thread : collector_threads_) {
-      thread.Join();
-    }
-  }
-
-  void Mark(std::string name) {
-    if (name == "benchmark") {
-      StartManager(std::move(name));
-    } else {
-      CollectManager();
-    }
-  }
-
- private:
-  void StartManager(std::string name) {
-    CollectManager();
-    done_ = std::make_shared<grpc_core::Notification>();
-    manager_thread_.emplace("latent_see_manager", [this, done = done_,
-                                                   name = std::move(name)]() {
-      LOG(INFO) << "Starting latent-see";
-      grpc_core::latent_see::Log::Get().TryPullEventsAndFlush(
-          [](absl::Span<const grpc_core::latent_see::Log::RecordedEvent>) {});
-      LOG(INFO) << "Started latent-see";
-      int step = 0;
-      // Once per second: kick off a thread to collate the latent-see data.
-      // Each one of these may fail under contention (that's ok, we just get
-      // a subset of the data).
-      while (!done->WaitForNotificationWithTimeout(absl::Seconds(1))) {
-        // Bound the number of collector threads outstanding.
-        while (collector_threads_.size() > 32) {
-          collector_threads_.front().Join();
-          collector_threads_.pop_front();
-        }
-        // Kick off a new thread
-        collector_threads_
-            .emplace_back("latent_see_collector",
-                          [step, name, this]() { Collect(step, name); })
-            .Start();
-        ++step;
-      }
-    });
-    manager_thread_->Start();
-  }
-
-  void CollectManager() {
-    if (manager_thread_.has_value()) {
-      done_->Notify();
-      manager_thread_->Join();
-      manager_thread_.reset();
-    }
-  }
-
-  void Collect(int step, std::string name) {
-    auto json = grpc_core::latent_see::Log::Get().TryGenerateJson();
-    if (!json.has_value()) {
-      LOG(INFO) << "Failed to generate latent_see.json (contention with "
-                   "another writer)";
-      return;
-    }
-    const std::string filename = absl::StrCat(
-        "latent_see_", role_, "_", name, "_", getpid(), "_", step, ".json");
-    LOG(INFO) << "Writing " << filename << " in " << get_current_dir_name();
-    FILE* f = fopen(filename.c_str(), "w");
-    if (f == nullptr) return;
-    fprintf(f, "%s", json->c_str());
-    fclose(f);
-  }
-
-  std::shared_ptr<grpc_core::Notification> done_;
-  std::optional<grpc_core::Thread> manager_thread_;
-  std::deque<grpc_core::Thread> collector_threads_;
-  const std::string role_;
-};
-#else
-class ScopedLatentSee final {
- public:
-  explicit ScopedLatentSee(const std::string&) {}
-  void Mark(const std::string&) {}
-};
-#endif
-
 class WorkerServiceImpl final : public WorkerService::Service {
  public:
   WorkerServiceImpl(int server_port, QpsWorker* worker)
@@ -277,7 +182,6 @@ class WorkerServiceImpl final : public WorkerService::Service {
 
   Status RunClientBody(ServerContext* /*ctx*/,
                        ServerReaderWriter<ClientStatus, ClientArgs>* stream) {
-    ScopedLatentSee latent_see("client");
     ClientArgs args;
     if (!stream->Read(&args)) {
       return Status(StatusCode::INVALID_ARGUMENT, "Couldn't read args");
@@ -303,7 +207,6 @@ class WorkerServiceImpl final : public WorkerService::Service {
         return Status(StatusCode::INVALID_ARGUMENT, "Invalid mark");
       }
       *status.mutable_stats() = client->Mark(args.mark().reset());
-      latent_see.Mark(args.mark().name());
       if (!stream->Write(status)) {
         return Status(StatusCode::UNKNOWN, "Client couldn't respond to mark");
       }
@@ -319,7 +222,6 @@ class WorkerServiceImpl final : public WorkerService::Service {
 
   Status RunServerBody(ServerContext* /*ctx*/,
                        ServerReaderWriter<ServerStatus, ServerArgs>* stream) {
-    ScopedLatentSee latent_see("server");
     ServerArgs args;
     if (!stream->Read(&args)) {
       return Status(StatusCode::INVALID_ARGUMENT, "Couldn't read server args");
@@ -353,7 +255,6 @@ class WorkerServiceImpl final : public WorkerService::Service {
         return Status(StatusCode::INVALID_ARGUMENT, "Invalid mark");
       }
       *status.mutable_stats() = server->Mark(args.mark().reset());
-      latent_see.Mark(args.mark().name());
       if (!stream->Write(status)) {
         return Status(StatusCode::UNKNOWN, "Server couldn't respond to mark");
       }
@@ -373,6 +274,10 @@ class WorkerServiceImpl final : public WorkerService::Service {
 QpsWorker::QpsWorker(int driver_port, int server_port,
                      const std::string& credential_type) {
   impl_ = std::make_unique<WorkerServiceImpl>(server_port, this);
+  latent_see_ =
+      std::make_unique<LatentSeeService>(LatentSeeService::Options()
+                                             .set_max_memory(1024 * 1024 * 1024)
+                                             .set_max_query_time(30.0));
   gpr_atm_rel_store(&done_, gpr_atm{0});
 
   std::unique_ptr<ServerBuilder> builder = CreateQpsServerBuilder();
@@ -383,6 +288,7 @@ QpsWorker::QpsWorker(int driver_port, int server_port,
         server_address,
         GetCredentialsProvider()->GetServerCredentials(credential_type));
   }
+  builder->RegisterService(latent_see_.get());
   builder->RegisterService(impl_.get());
 
   server_ = builder->BuildAndStart();
