@@ -46,7 +46,6 @@ EXTERNAL_DEPS = {
     "absl/base/call_once.h": "absl/base",
     "absl/base/config.h": "absl/base:config",
     # TODO(ctiller) remove this
-    "absl/base/internal/endian.h": "absl/base:endian",
     "absl/base/no_destructor.h": "absl/base:no_destructor",
     "absl/base/thread_annotations.h": "absl/base:core_headers",
     "absl/container/flat_hash_map.h": "absl/container:flat_hash_map",
@@ -165,13 +164,13 @@ EXTERNAL_DEPS = {
     "openssl/x509.h": "libcrypto",
     "openssl/x509v3.h": "libcrypto",
     "re2/re2.h": "re2",
-    "upb/base/status.hpp": "@com_google_protobuf//upb:base",
-    "upb/base/string_view.h": "@com_google_protobuf//upb:base",
-    "upb/message/map.h": "@com_google_protobuf//upb:message",
+    "upb/base/status.hpp": "@com_google_protobuf//upb/base",
+    "upb/base/string_view.h": "@com_google_protobuf//upb/base",
+    "upb/message/map.h": "@com_google_protobuf//upb/message",
     "upb/reflection/def.h": "upb_reflection",
     "upb/json/encode.h": "upb_json_lib",
-    "upb/mem/arena.h": "@com_google_protobuf//upb:mem",
-    "upb/mem/arena.hpp": "@com_google_protobuf//upb:mem",
+    "upb/mem/arena.h": "@com_google_protobuf//upb/mem",
+    "upb/mem/arena.hpp": "@com_google_protobuf//upb/mem",
     "upb/text/encode.h": "upb_textformat_lib",
     "upb/reflection/def.hpp": "upb_reflection",
     "xxhash.h": "xxhash",
