@@ -142,6 +142,7 @@ LANG_RELEASE_MATRIX = {
             ("v1.71.0", ReleaseInfo()),
             ("v1.72.0", ReleaseInfo()),
             ("v1.73.0", ReleaseInfo()),
+            ("v1.74.0", ReleaseInfo()),
         ]
     ),
     "go": OrderedDict(
@@ -319,6 +320,7 @@ LANG_RELEASE_MATRIX = {
             ("v1.71.3", ReleaseInfo()),
             ("v1.72.2", ReleaseInfo()),
             ("v1.73.0", ReleaseInfo()),
+            ("v1.74.2", ReleaseInfo()),
         ]
     ),
     "java": OrderedDict(
@@ -470,6 +472,7 @@ LANG_RELEASE_MATRIX = {
             ("v1.70.0", ReleaseInfo()),
             ("v1.71.0", ReleaseInfo()),
             ("v1.72.0", ReleaseInfo()),
+            ("v1.73.0", ReleaseInfo()),
         ]
     ),
     "python": OrderedDict(
@@ -870,6 +873,12 @@ LANG_RELEASE_MATRIX = {
                     runtimes=["python"], testcases_file="python__master"
                 ),
             ),
+            (
+                "v1.74.0",
+                ReleaseInfo(
+                    runtimes=["python"], testcases_file="python__master"
+                ),
+            ),
         ]
     ),
     "node": OrderedDict(
@@ -976,6 +985,7 @@ LANG_RELEASE_MATRIX = {
             ("v1.71.0", ReleaseInfo()),
             ("v1.72.0", ReleaseInfo()),
             ("v1.73.0", ReleaseInfo()),
+            ("v1.74.0", ReleaseInfo()),
         ]
     ),
     "php": OrderedDict(
@@ -1045,6 +1055,7 @@ LANG_RELEASE_MATRIX = {
             ("v1.71.0", ReleaseInfo()),
             ("v1.72.0", ReleaseInfo()),
             ("v1.73.0", ReleaseInfo()),
+            ("v1.74.0", ReleaseInfo()),
         ]
     ),
     "csharp": OrderedDict(
